#!/usr/bin/env python3

import sys
from PyQt5 import QtWidgets, QtCore, QtGui

numRows = 5000

# Variant 1: Template for calling setStyleSheet (static CSS, dynamically changed)
styleTemplate = """
QProgressBar::chunk {
    background-color: %s;
    width: 1px;
}
QProgressBar {
    background-color: %s;
    border: none;
    text-align: center;
}
"""

# Variant 2: Style sheet with attributes for dynamic properties (set once, properties dynamically changed)
dynStyle = """
QProgressBar { border: none; text-align: center; }
QProgressBar::chunk { width: 1px; }
QProgressBar[state="low"] { background-color: lightyellow; }
QProgressBar[state="low"]::chunk { background-color: yellow;}
QProgressBar[state="high"] { background-color: orange; }
QProgressBar[state="high"]::chunk { background-color: red; }
"""

# Variant 3: use QPalette instead of style sheets
paletteLow = QtGui.QPalette()
paletteLow.setColor(QtGui.QPalette.Highlight, QtGui.QColor("yellow").darker(120))
paletteLow.setColor(QtGui.QPalette.Base, QtGui.QColor("lightyellow"))
paletteLow.setColor(QtGui.QPalette.Text, QtGui.QColor("black"))
paletteLow.setColor(QtGui.QPalette.HighlightedText, QtGui.QColor("black"))
paletteHigh = QtGui.QPalette()
paletteHigh.setColor(QtGui.QPalette.Highlight, QtGui.QColor("red"))
paletteHigh.setColor(QtGui.QPalette.Base, QtGui.QColor("orange"))
paletteHigh.setColor(QtGui.QPalette.Text, QtGui.QColor("black"))
paletteHigh.setColor(QtGui.QPalette.HighlightedText, QtGui.QColor("black"))


# Variant 3: Use an item delegate for a QTableView to paint the column as a progress bar
class ProgressDelegate(QtWidgets.QStyledItemDelegate):
    def __init__(self, widget, style):
        super().__init__(widget)
        self.widget = widget
        self.style = style

    def paint(self, painter, option, index):
        progress = index.data(QtCore.Qt.DisplayRole)
        opt = QtWidgets.QStyleOptionProgressBar()
        opt.rect = option.rect
        opt.minimum = 0
        opt.maximum = 100
        opt.progress = progress
        if progress < 0:
            opt.progress = 0
            opt.maximum = 0
            opt.styleObject = self.widget.viewport()
        opt.textVisible = progress >= 0
        opt.palette = paletteLow if progress < 50 else paletteHigh
        opt.text = "{}%".format(progress)
        self.style.drawControl(QtWidgets.QStyle.CE_ProgressBar, opt, painter)


class TableModel(QtCore.QAbstractTableModel):
    def __init__(self):
        super().__init__()
        self.tickOfs = -1

    def data(self, index, role):
        if role == QtCore.Qt.DisplayRole:
            r, c = index.row(), index.column()
            if c == 0:
                return ["Foo", "Bar", "Baz"][r%3]
            return (r+self.tickOfs) % 102 - 1

    def headerData(self, section, orientation, role):
        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
            return ["Text", "Progress"][section]
        return super().headerData(section, orientation, role)

    def rowCount(self, index):
        return numRows

    def columnCount(self, index):
        return 2

    def tickValues(self):
        self.tickOfs += 1
        self.dataChanged.emit(self.index(0,0), self.index(1, numRows-1))


class Window(QtWidgets.QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_Window()
        self.ui.setupUi(self)
        self.ui.updateLeftBtn.clicked.connect(self.on_updateLeftBtn_clicked)
        self.ui.updateMiddleBtn.clicked.connect(self.on_updateMiddleBtn_clicked)
        self.ui.updateRightBtn.clicked.connect(self.on_updateRightBtn_clicked)
        self.tickOfs = 0
        self.initTable(self.ui.tableCssTemplate)
        self.initTable(self.ui.tableDynProp)
        self.ui.tableView.verticalHeader().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
        self.updateStylesLeft()
        self.updateStylesMiddle()
        self.model = TableModel()
        self.delegate = ProgressDelegate(self.ui.tableView, QtWidgets.QStyleFactory.create("fusion"))
        self.ui.tableView.setItemDelegateForColumn(1, self.delegate)
        self.ui.tableView.setModel(self.model)
        self.tickValuesLeft()
        self.tickValuesMiddle()
        self.tickValuesRight()
        self.initStyleDynProp(self.ui.tableDynProp)

    # Variant 1: Generates a style from a template, then sets it with setStyleSheet.
    # This is slow because of setStyleSheet
    def updateStyleCssTemplate(self, tbl):
        for r in range(numRows):
            bar = tbl.cellWidget(r, 1)
            col, back = ("yellow", "lightyellow") if bar.value() < 50 else ("red", "orange")
            bar.setStyleSheet(styleTemplate % (col, back))

    # Variant 2: Only update a dynamic property corresponding to the CSS attribute
    # This is supposed to be faster, because it avoids setStyleSheet. However, this doesn't get
    # updates with a simple repaint. Instead, this unpolish/polish hack has to be used, which
    # is even slower.
    def updateStyleDynProp(self, tbl):
        for r in range(numRows):
            bar = tbl.cellWidget(r, 1)
            bar.setProperty("state", "low" if bar.value() < 50 else "high")
            bar.style().unpolish(bar)
            #bar.style().polish(bar)

    def on_updateLeftBtn_clicked(self):
        self.tickValuesLeft()
        self.updateStylesLeft()

    def on_updateMiddleBtn_clicked(self):
        self.tickValuesMiddle()
        self.updateStylesMiddle()

    def on_updateRightBtn_clicked(self):
        self.tickValuesRight()
        self.updateRight()

    def updateStylesLeft(self):
        t = QtCore.QElapsedTimer()
        t.start()
        self.updateStyleCssTemplate(self.ui.tableCssTemplate)
        self.ui.labelCssTemplate.setText(f"Time {t.elapsed()} ms")

    def updateStylesMiddle(self):
        t = QtCore.QElapsedTimer()
        t.start()
        self.updateStyleDynProp(self.ui.tableDynProp)
        self.ui.labelDynProp.setText(f"Time {t.elapsed()} ms")

    def updateRight(self):
        t = QtCore.QElapsedTimer()
        t.start()
        self.ui.tableView.viewport().update()
        self.ui.labelView.setText(f"Time {t.elapsed()} ms")

    def tickValuesLeft(self):
        self.tickValues(self.ui.tableCssTemplate)

    def tickValuesMiddle(self):
        self.tickValues(self.ui.tableDynProp)

    def tickValuesRight(self):
        self.model.tickValues()

    def tickValues(self, tbl):
        for r in range(numRows):
            bar = tbl.cellWidget(r, 1)
            p = (r+self.tickOfs) % 102 - 1
            if p < 0:
                bar.setRange(0, 0)
            else:
                bar.setRange(0, 100)
            bar.setValue(p)
        self.tickOfs += 1

    def initTable(self, tbl):
        tbl.setRowCount(numRows)
        for r in range(numRows):
            tbl.setItem(r, 0, QtWidgets.QTableWidgetItem(["Foo", "Bar", "Baz"][r%3]))
            tbl.setCellWidget(r, 1, QtWidgets.QProgressBar())
        tbl.verticalHeader().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
        tbl.horizontalHeader().setSectionResizeMode(QtWidgets.QHeaderView.Stretch)

    def initStyleDynProp(self, tbl):
        for r in range(numRows):
            bar = tbl.cellWidget(r, 1)
            bar.setStyleSheet(dynStyle)

# Generated UI code
class Ui_Window(object):
    def setupUi(self, Window):
        Window.resize(950, 300)
        self.gridLayout = QtWidgets.QGridLayout(Window)
        self.labelCssTemplate = QtWidgets.QLabel(Window)
        self.gridLayout.addWidget(self.labelCssTemplate, 2, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(Window)
        self.gridLayout.addWidget(self.label_2, 0, 1, 1, 1)
        self.tableDynProp = QtWidgets.QTableWidget(Window)
        self.tableDynProp.setColumnCount(2)
        self.tableDynProp.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableDynProp.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableDynProp.setHorizontalHeaderItem(1, item)
        self.tableDynProp.horizontalHeader().setStretchLastSection(True)
        self.gridLayout.addWidget(self.tableDynProp, 1, 1, 1, 1)
        self.label = QtWidgets.QLabel(Window)
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.tableCssTemplate = QtWidgets.QTableWidget(Window)
        self.tableCssTemplate.setColumnCount(2)
        self.tableCssTemplate.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableCssTemplate.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableCssTemplate.setHorizontalHeaderItem(1, item)
        self.tableCssTemplate.horizontalHeader().setStretchLastSection(True)
        self.gridLayout.addWidget(self.tableCssTemplate, 1, 0, 1, 1)
        self.labelDynProp = QtWidgets.QLabel(Window)
        self.gridLayout.addWidget(self.labelDynProp, 2, 1, 1, 1)
        self.updateMiddleBtn = QtWidgets.QPushButton(Window)
        self.gridLayout.addWidget(self.updateMiddleBtn, 3, 1, 1, 1)
        self.tableView = QtWidgets.QTableView(Window)
        self.tableView.horizontalHeader().setStretchLastSection(True)
        self.gridLayout.addWidget(self.tableView, 1, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(Window)
        self.gridLayout.addWidget(self.label_3, 0, 2, 1, 1)
        self.updateRightBtn = QtWidgets.QPushButton(Window)
        self.gridLayout.addWidget(self.updateRightBtn, 3, 2, 1, 1)
        self.updateLeftBtn = QtWidgets.QPushButton(Window)
        self.gridLayout.addWidget(self.updateLeftBtn, 3, 0, 1, 1)
        self.labelView = QtWidgets.QLabel(Window)
        self.gridLayout.addWidget(self.labelView, 2, 2, 1, 1)

        self.retranslateUi(Window)
        #QtCore.QMetaObject.connectSlotsByName(Window)

    def retranslateUi(self, Window):
        _translate = QtCore.QCoreApplication.translate
        self.label_2.setText(_translate("Window", "Static CSS with dynamic properties ([un]polish)"))
        item = self.tableDynProp.horizontalHeaderItem(0)
        item.setText(_translate("Window", "Text"))
        item = self.tableDynProp.horizontalHeaderItem(1)
        item.setText(_translate("Window", "Progress"))
        self.label.setText(_translate("Window", "Dynamically set CSS from template (setStyleSheet)"))
        item = self.tableCssTemplate.horizontalHeaderItem(0)
        item.setText(_translate("Window", "Text"))
        item = self.tableCssTemplate.horizontalHeaderItem(1)
        item.setText(_translate("Window", "Progress"))
        self.updateMiddleBtn.setText(_translate("Window", "Update (middle)"))
        self.label_3.setText(_translate("Window", "TableView"))
        self.updateRightBtn.setText(_translate("Window", "Update (right)"))
        self.updateLeftBtn.setText(_translate("Window", "Update (left)"))



def main():
    app = QtWidgets.QApplication(sys.argv)
    mainwin = Window()
    mainwin.show()
    app.exec_()

if __name__ == '__main__':
    main()